import 'dart:convert';
import 'dart:io';

import 'package:deuna_checkout_widget/src/deuna_checkout_configuration.dart';
import 'package:flutter/widgets.dart';
import 'package:webview_flutter/webview_flutter.dart';

/// {@template deuna_checkout_widget.DeUnaCheckout}
/// [DeUnaCheckout] is a widget that allows you to display a webview with the
/// [DeUnaCheckoutConfiguration] provided.
/// {@endtemplate}
class DeUnaCheckout extends StatefulWidget {
  /// {@macro deuna_checkout_widget.DeUnaCheckout}
  const DeUnaCheckout({
    Key? key,
    required this.orderToken,
    this.configuration,
  }) : super(key: key);

  /// The order token to be used to create the payment.
  final String orderToken;

  /// Optional configuaration. If not provided, the configuration will be
  /// read using the [DeUnaCheckoutConfiguration].of(context) method.
  final DeUnaCheckoutData? configuration;

  @override
  State<DeUnaCheckout> createState() => _DeUnaCheckoutState();
}

class _DeUnaCheckoutState extends State<DeUnaCheckout> {
  late WebViewController _controller;

  @override
  void initState() {
    super.initState();
    // Enable virtual display.
    if (Platform.isAndroid) WebView.platform = AndroidWebView();
  }

  DeUnaCheckoutData get config =>
      widget.configuration ?? DeUnaCheckoutConfiguration.of(context);

  void injectProps() {
    final _config = <String, dynamic>{
      'apiKey': config.apiKey,
      'orderToken': widget.orderToken,
      if (config.analyticsConfig != null)
        'analytics': config.analyticsConfig!.toJson(),
    };
    final configString = jsonEncode(_config);
    _controller.runJavascript(
      '''
(function() {
  window.xprops = $configString;
})();
''',
    );
  }

  @override
  Widget build(BuildContext context) {
    return WebView(
      javascriptMode: JavascriptMode.unrestricted,
      debuggingEnabled: true,
      javascriptChannels: <JavascriptChannel>{
        JavascriptChannel(
          name: 'DeUnaCheckoutChannel',
          onMessageReceived: (JavascriptMessage message) {
            if (message.message == 'done') {
              Navigator.pop(context);
            }
          },
        ),
        JavascriptChannel(
          name: 'PrintChannel',
          onMessageReceived: (JavascriptMessage message) {
            assert(
              () {
                debugPrint(message.message);
                return true;
              }(),
              'Only for debug purpose',
            );
          },
        ),
      },
      initialUrl: config.env.url,
      onPageStarted: (url) => injectProps(),
      onWebViewCreated: (controller) => _controller = controller,
    );
  }
}

extension _HtmlInjector on WebViewController {
  //Future<String> readXprops() {
  //  return runJavascriptReturningResult('JSON.stringify(window.xprops)');
  //}

}
